@@ -58,11 +58,29 @@ static void
 control_connection_died_cb (gpointer connection,
 			    gpointer user_data)
 {
+	CORBA_Object cntl;
+	CORBA_Environment ev;
 	BonoboControlFrame *frame = BONOBO_CONTROL_FRAME (user_data);
 
 	g_return_if_fail (frame != NULL);
 
 	dprintf ("The remote control end died unexpectedly");
+
+	CORBA_exception_init (&ev);
+
+	cntl = CORBA_Object_duplicate (frame->priv->control, &ev);
+
+	bonobo_control_frame_bind_to_control (
+		frame, CORBA_OBJECT_NIL, NULL);
+
+	CORBA_exception_set_system (
+		&ev, ex_CORBA_COMM_FAILURE, CORBA_COMPLETED_YES);
+
+	bonobo_object_check_env (BONOBO_OBJECT (frame), cntl, &ev);
+
+	CORBA_Object_release (cntl, &ev);
+
+	CORBA_exception_free (&ev);
 }
 
 static Bonobo_Gdk_WindowId
@@ -134,20 +152,23 @@ impl_Bonobo_ControlFrame_notifyActivated (PortableServer_Servant  servant,
 	BonoboControlFrame *frame = BONOBO_CONTROL_FRAME (
 		bonobo_object_from_servant (servant));
 
-	gtk_signal_emit (GTK_OBJECT (frame),
-			 control_frame_signals [ACTIVATED], state);
+	g_signal_emit (frame, control_frame_signals [ACTIVATED], 0, state);
 }
 
 static void
 impl_Bonobo_ControlFrame_queueResize (PortableServer_Servant  servant,
 				      CORBA_Environment      *ev)
 {
-	/*
-	 * Nothing.
-	 *
-	 * In the Gnome implementation of Bonobo, all size negotiation
-	 * is handled by GtkPlug/GtkSocket for us.
-	 */
+	/* Supposedly Gtk+ handles size negotiation properly for us */
+/*	BonoboSocket *socket;
+	BonoboControlFrame *frame;
+
+	frame = BONOBO_CONTROL_FRAME (bonobo_object (servant));
+
+	if ((socket = frame->priv->socket)) {
+		GTK_SOCKET (socket)->have_size = FALSE;
+		gtk_widget_queue_resize (GTK_WIDGET (socket));
+		} */
 }
 
 static void
@@ -158,9 +179,9 @@ impl_Bonobo_ControlFrame_activateURI (PortableServer_Servant  servant,
 {
 	BonoboControlFrame *frame = BONOBO_CONTROL_FRAME (bonobo_object_from_servant (servant));
 
-	gtk_signal_emit (GTK_OBJECT (frame),
-			 control_frame_signals [ACTIVATE_URI],
-			 (const char *) uri, (gboolean) relative);
+	g_signal_emit (frame,
+		       control_frame_signals [ACTIVATE_URI], 0,
+		       (const char *) uri, (gboolean) relative);
 }
 
 #ifdef DEBUG_CONTROL
@@ -404,9 +425,6 @@ bonobo_control_frame_class_init (BonoboControlFrameClass *klass)
 	epv->notifyActivated      = impl_Bonobo_ControlFrame_notifyActivated;
 	epv->queueResize          = impl_Bonobo_ControlFrame_queueResize;
 	epv->activateURI          = impl_Bonobo_ControlFrame_activateURI;
-	epv->unImplemented        = NULL;
-	epv->unImplemented2       = NULL;
-	epv->unImplemented3       = NULL;
 }
 
 static void
@@ -722,7 +740,7 @@ bonobo_control_frame_bind_to_control (BonoboControlFrame *frame,
 	} else
 		ev = opt_ev;
 
-	g_object_ref (G_OBJECT (frame));
+	g_object_ref (frame);
 
 	if (frame->priv->control != CORBA_OBJECT_NIL) {
 		if (!frame->priv->inproc_control)
@@ -762,7 +780,7 @@ bonobo_control_frame_bind_to_control (BonoboControlFrame *frame,
 		bonobo_control_frame_get_remote_window (frame, ev);
 	}
 
-	g_object_unref (G_OBJECT (frame));
+	g_object_unref (frame);
 
 	if (!opt_ev)
 		CORBA_exception_free (&tmp_ev);
@@ -832,8 +850,8 @@ bonobo_control_frame_set_propbag (BonoboControlFrame  *frame,
 	if (old_pb == propbag)
 		return;
 
-	frame->priv->propbag = bonobo_object_ref (BONOBO_OBJECT (propbag));
-	bonobo_object_unref (BONOBO_OBJECT (old_pb));
+	frame->priv->propbag = bonobo_object_ref ((BonoboObject *) propbag);
+	bonobo_object_unref ((BonoboObject *) old_pb);
 }
 
 /**
@@ -882,6 +900,7 @@ bonobo_control_frame_get_control_property_bag (BonoboControlFrame *frame,
 
 	control = frame->priv->control;
 
+	/* FIXME: we could cache this here - is it called a lot ? */
 	pbag = Bonobo_Control_getProperties (control, real_ev);
 
 	if (BONOBO_EX (real_ev)) {
@@ -1016,7 +1035,7 @@ bonobo_control_frame_set_socket (BonoboControlFrame *frame,
 	old_socket = (BonoboSocket *) frame->priv->socket;
 
 	if (socket) {
-		frame->priv->socket = g_object_ref (G_OBJECT (socket));
+		frame->priv->socket = g_object_ref (socket);
 		bonobo_socket_set_control_frame (socket, frame);
 	} else
 		frame->priv->socket = NULL;
@@ -1024,7 +1043,7 @@ bonobo_control_frame_set_socket (BonoboControlFrame *frame,
 	if (old_socket) {
 		bonobo_socket_set_control_frame (
 			BONOBO_SOCKET (old_socket), NULL);
-		g_object_unref (G_OBJECT (old_socket));
+		g_object_unref (old_socket);
 	}
 }
 
@@ -1040,6 +1059,38 @@ BonoboUIComponent *
 bonobo_control_frame_get_popup_component (BonoboControlFrame *control_frame,
 					  CORBA_Environment  *opt_ev)
 {
-	/* FIXME: impl. me */
-	return CORBA_OBJECT_NIL;
+	CORBA_Environment *ev, tmp_ev;
+	BonoboUIComponent *ui_component;
+	Bonobo_UIContainer popup_container;
+
+	g_return_val_if_fail (BONOBO_IS_CONTROL_FRAME (control_frame), NULL);
+
+	if (control_frame->priv->control == CORBA_OBJECT_NIL)
+		return NULL;
+
+	ui_component = bonobo_ui_component_new_default ();
+
+	if (!opt_ev) {
+		CORBA_exception_init (&tmp_ev);
+		ev = &tmp_ev;
+	} else
+		ev = opt_ev;
+
+	popup_container = Bonobo_Control_getPopupContainer (
+		control_frame->priv->control, ev);
+
+	bonobo_ui_component_set_container (ui_component, popup_container, ev);
+
+	Bonobo_Unknown_unref (popup_container, ev);
+
+	if (ev->_major != CORBA_NO_EXCEPTION) {
+		bonobo_object_unref (BONOBO_OBJECT (ui_component));
+		ui_component = NULL;
+	}
+
+	if (!opt_ev) {
+		CORBA_exception_free (ev);
+	}
+
+	return ui_component;
 }
