@@ -8,6 +8,11 @@
  * Copyright 2000,2001 Ximian, Inc.
  */
 
+/* FIXME: bonobo_ui_engine_update should take
+ * a BonoboUINode *, which we can walk up from
+ * looking for cleanliness & then re-building
+ * from there on down */
+
 #include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -29,6 +34,12 @@
 
 #include <bonobo/bonobo-ui-node-private.h>
 
+#ifdef LOWLEVEL_DEBUG
+#  define ACCESS(str) access (str, 0)
+#else
+#  define ACCESS(str)
+#endif
+
 /* Various debugging output defines */
 #undef STATE_SYNC_DEBUG
 #undef WIDGET_SYNC_DEBUG
@@ -39,10 +50,12 @@
 static GObjectClass *parent_class = NULL;
 
 static GQuark id_id        = 0;
+static GQuark cmd_id       = 0;
 static GQuark verb_id      = 0;
 static GQuark name_id      = 0;
 static GQuark state_id     = 0;
 static GQuark hidden_id    = 0;
+static GQuark commands_id  = 0;
 static GQuark sensitive_id = 0;
 
 enum {
@@ -83,10 +96,12 @@ find_sync_for_node (BonoboUIEngine *engine,
 	GSList       *l;
 	BonoboUISync *ret = NULL;
 
-	if (!node) {
-/*		fprintf (stderr, "Could not find sync for some path\n");*/
+	if (!node)
+		return NULL;
+
+	if (node->name_id == cmd_id ||
+	    node->name_id == commands_id)
 		return NULL;
-	}
 
 	for (l = engine->priv->syncs; l; l = l->next) {
 		if (bonobo_ui_sync_can_handle (l->data, node)) {
@@ -97,7 +112,7 @@ find_sync_for_node (BonoboUIEngine *engine,
 
 	if (ret) {
 /*		fprintf (stderr, "Found sync '%s' for path '%s'\n",
-			 gtk_type_name (GTK_CLASS_TYPE (GTK_OBJECT_GET_CLASS (ret))),
+			 gtk_type_name (G_TYPE_FROM_CLASS (GTK_OBJECT_GET_CLASS (ret))),
 			 bonobo_ui_xml_make_path (node));*/
 		return ret;
 	}
@@ -1208,25 +1223,17 @@ bonobo_ui_engine_object_get (BonoboUIEngine    *engine,
 	return BONOBO_UI_ERROR_OK;
 }
 
-static char *
-get_parent_path (const char *path)
+static int
+find_last_slash (const char *path)
 {
 	int i, last_slash = 0;
-	char *ret;
 
 	for (i = 0; path [i]; i++) {
 		if (path [i] == '/')
 			last_slash = i;
 	}
 
-	if (!last_slash)
-		return NULL;
-
-	ret = g_malloc (last_slash + 1);
-	memcpy (ret, path, last_slash);
-	ret [last_slash] = '\0';
-
-	return ret;
+	return last_slash;
 }
 
 /**
@@ -1249,9 +1256,10 @@ bonobo_ui_engine_xml_set_prop (BonoboUIEngine *engine,
 			       const char     *value,
 			       const char     *component)
 {
-	char *parent_path;
-	BonoboUINode *copy;
+	char *cmp_name;
+	const char *old_value;
 	BonoboUINode *original;
+	NodeInfo     *info;
 	
 	g_return_val_if_fail (BONOBO_IS_UI_ENGINE (engine), 
 			      BONOBO_UI_ERROR_BAD_PARAM);
@@ -1261,15 +1269,47 @@ bonobo_ui_engine_xml_set_prop (BonoboUIEngine *engine,
 	if (!original) 
 		return BONOBO_UI_ERROR_INVALID_PATH;
 
-	copy = bonobo_ui_node_new (bonobo_ui_node_get_name (original));
-	bonobo_ui_node_copy_attrs (original, copy);
-	bonobo_ui_node_set_attr (copy, property, value);
+	info = bonobo_ui_xml_get_data (engine->priv->tree, original);
+	cmp_name = sub_component_cmp_name (engine, component);
+
+	if (info->parent.id == cmp_name) {
+		old_value = bonobo_ui_node_peek_attr (original, property);
+		if (!old_value && !value)
+			return BONOBO_UI_ERROR_OK;
+		
+		else if (old_value && value && !strcmp (old_value, value))
+			return BONOBO_UI_ERROR_OK;
+
+		else {
+			bonobo_ui_node_set_attr (original, property, value);
+			bonobo_ui_xml_set_dirty (engine->priv->tree, original);
+
+			bonobo_ui_engine_update (engine);
+		}
+	} else {
+		int last_slash;
+		char *parent_path;
+		BonoboUINode *copy;
+
+		copy = bonobo_ui_node_new (
+			bonobo_ui_node_get_name (original));
+
+		bonobo_ui_node_copy_attrs (original, copy);
+		bonobo_ui_node_set_attr (copy, property, value);
+
+		last_slash = find_last_slash (path);
+
+		parent_path = g_alloca (last_slash + 1);
+		memcpy (parent_path, path, last_slash);
+		parent_path [last_slash] = '\0';
+
+		bonobo_ui_xml_merge (
+			engine->priv->tree, parent_path,
+			copy, cmp_name);
+
+		bonobo_ui_engine_update (engine);
+	}
 
-	parent_path = get_parent_path (path);
-	bonobo_ui_engine_xml_merge_tree (
-		engine, parent_path, copy, component);
-	g_free (parent_path);
-	
 	return BONOBO_UI_ERROR_OK;
 }
 
@@ -1427,7 +1467,7 @@ real_exec_verb (BonoboUIEngine *engine,
 	g_return_if_fail (component_name != NULL);
 	g_return_if_fail (BONOBO_IS_UI_ENGINE (engine));
 
-	g_object_ref (G_OBJECT (engine));
+	g_object_ref (engine);
 
 	component = sub_component_objref (engine, component_name);
 
@@ -1453,7 +1493,7 @@ real_exec_verb (BonoboUIEngine *engine,
 		CORBA_exception_free (&ev);
 	}
 
-	g_object_unref (G_OBJECT (engine));
+	g_object_unref (engine);
 }
 
 static void
@@ -1662,7 +1702,7 @@ real_emit_ui_event (BonoboUIEngine *engine,
 	if (!component_name) /* Auto-created entry, no-one can listen to it */
 		return;
 
-	g_object_ref (G_OBJECT (engine));
+	g_object_ref (engine);
 
 	component = sub_component_objref (engine, component_name);
 
@@ -1688,7 +1728,7 @@ real_emit_ui_event (BonoboUIEngine *engine,
 		CORBA_exception_free (&ev);
 	}
 
-	g_object_unref (G_OBJECT (engine));
+	g_object_unref (engine);
 }
 
 static void
@@ -1708,7 +1748,7 @@ impl_emit_event_on (BonoboUIEngine *engine,
 	data = bonobo_ui_xml_get_data (NULL, node);
 	g_return_if_fail (data != NULL);
 
-	g_object_ref (G_OBJECT (engine));
+	g_object_ref (engine);
 
 	component_id = g_strdup (data->id);
 	real_id      = g_strdup (id);
@@ -1720,7 +1760,7 @@ impl_emit_event_on (BonoboUIEngine *engine,
 			    Bonobo_UIComponent_STATE_CHANGED,
 			    state);
 
-	g_object_unref (G_OBJECT (engine));
+	g_object_unref (engine);
 
 	g_free (component_id);
 	g_free (real_id);
@@ -1746,12 +1786,12 @@ bonobo_ui_engine_dispose (BonoboUIEngine *engine)
 	bonobo_ui_preferences_remove_engine (engine);
 
 	if (priv->config) {
-		g_object_unref (G_OBJECT (priv->config));
+		g_object_unref (priv->config);
 		priv->config = NULL;
 	}
 
 	if (priv->tree) {
-		g_object_unref (G_OBJECT (priv->tree));
+		g_object_unref (priv->tree);
 		priv->tree = NULL;
 	}
 
@@ -1761,7 +1801,7 @@ bonobo_ui_engine_dispose (BonoboUIEngine *engine)
 		NULL);
 
 	for (l = priv->syncs; l; l = l->next)
-		g_object_unref (G_OBJECT (l->data));
+		g_object_unref (l->data);
 	g_slist_free (priv->syncs);
 	priv->syncs = NULL;
 
@@ -1800,10 +1840,12 @@ class_init (BonoboUIEngineClass *engine_class)
 	parent_class = g_type_class_peek_parent (engine_class);
  
  	id_id        = g_quark_from_static_string ("id");
+	cmd_id       = g_quark_from_static_string ("cmd");
  	verb_id      = g_quark_from_static_string ("verb");
  	name_id      = g_quark_from_static_string ("name");
 	state_id     = g_quark_from_static_string ("state");
  	hidden_id    = g_quark_from_static_string ("hidden");
+	commands_id  = g_quark_from_static_string ("commands");
  	sensitive_id = g_quark_from_static_string ("sensitive");
 
 	object_class = G_OBJECT_CLASS (engine_class);
@@ -2441,16 +2483,16 @@ bonobo_ui_engine_update (BonoboUIEngine *engine)
 	if (engine->priv->frozen || !engine->priv->tree)
 		return;
 
-	access ("Bonobo: UI engine update - start", 0);
+	ACCESS ("Bonobo: UI engine update - start");
 
 	for (l = engine->priv->syncs; l; l = l->next)
 		bonobo_ui_sync_stamp_root (l->data);
 
-	access ("Bonobo: UI engine update - after stamp", 0);
+	ACCESS ("Bonobo: UI engine update - after stamp");
 
 	move_dirt_cmd_to_widget (engine);
 
-	access ("Bonobo: UI engine update - after dirt transfer", 0);
+	ACCESS ("Bonobo: UI engine update - after dirt transfer");
 
 /*	bonobo_ui_engine_dump (priv->win, "before update");*/
 
@@ -2466,15 +2508,15 @@ bonobo_ui_engine_update (BonoboUIEngine *engine)
 		bonobo_ui_engine_update_node (engine, sync, node);
 	}
 
-	access ("Bonobo: UI engine update - after update nodes", 0);
+	ACCESS ("Bonobo: UI engine update - after update nodes");
 
 	update_commands_state (engine);
 
-	access ("Bonobo: UI engine update - after cmd state", 0);
+	ACCESS ("Bonobo: UI engine update - after cmd state");
 
 	process_state_updates (engine);
 
-	access ("Bonobo: UI engine update - end", 0);
+	ACCESS ("Bonobo: UI engine update - end");
 
 /*	bonobo_ui_engine_dump (priv->win, "after update");*/
 }
