@@ -34,6 +34,9 @@
 
 static GObjectClass *parent_class = NULL;
 
+static GQuark dockitem_id = 0;
+static GQuark toolitem_id = 0;
+
 #define PARENT_TYPE bonobo_ui_sync_get_type ()
 
 static GdkPixbuf *
@@ -529,12 +532,12 @@ do_config_popup (BonoboUIEngineConfig *config,
 		"<popups>"
 		"<popup>"
 		"<submenu label=\"%s\">"
-		"<menuitem verb=\"LookBoth\" label=\"%s\" set=\"both\""
-		 "type=\"radio\" group=\"look\"/>"
-		"<menuitem verb=\"LookIcon\" label=\"%s\" set=\"icon\""
-		 "type=\"radio\" group=\"look\"/>"
-		"<menuitem verb=\"LookText\" label=\"%s\" set=\"text\""
-		 "type=\"radio\" group=\"look\"/>"
+		"<menuitem verb=\"LookBoth\" label=\"%s\" set=\"both\" "
+		"type=\"radio\" group=\"look\"/>"
+		"<menuitem verb=\"LookIcon\" label=\"%s\" set=\"icon\" "
+		"type=\"radio\" group=\"look\"/>"
+		"<menuitem verb=\"LookText\" label=\"%s\" set=\"text\" "
+		"type=\"radio\" group=\"look\"/>"
 		"</submenu>"
 		"<separator/>"
 		"<menuitem verb=\"Tip\" label=\"%s\" set=\"%d\"/>"
@@ -809,7 +812,13 @@ static gboolean
 impl_bonobo_ui_sync_toolbar_can_handle (BonoboUISync *sync,
 					BonoboUINode *node)
 {
-	return bonobo_ui_node_has_name (node, "dockitem");
+	if (!dockitem_id) {
+		dockitem_id = g_quark_from_static_string ("dockitem");
+		toolitem_id = g_quark_from_static_string ("toolitem");
+	}
+
+	return (node->name_id == dockitem_id ||
+		node->name_id == toolitem_id);
 }
 
 static void
